@@ -12,11 +12,15 @@ Test the netcdf-4 data building process.
 #include <stdio.h>
 #include "netcdf.h"
 
+#define DEBUG
+
 static void
 fail(int code)
 {
-    if(code != NC_NOERR)
+    if(code != NC_NOERR) {
 	fprintf(stderr,"***Fail: %s\n",nc_strerror(code));
+	fflush(stderr);
+    }
     exit((code==NC_NOERR?EXIT_SUCCESS:EXIT_FAILURE));
 }
 
@@ -37,10 +41,17 @@ main(int argc, char** argv)
     }
 
     /* build the url */
-    snprintf(url,sizeof(url),"file://%s#dap4&debug=copy&substratename=%s",argv[0],argv[1]);
+    snprintf(url,sizeof(url),"file://%s#dap4&debug=copy",argv[0]);
+    if(argc >= 3) {
+        strlcat(url,"&substratename=",sizeof(url));
+	strlcat(url,argv[1],sizeof(url));
+    }
+#ifdef DEBUG
+    strlcat(url,"&log",sizeof(url));
+#endif
 
 #ifdef DEBUG
-    fprintf(stderr,"t_dmrbuild %s -> %s\n",url,outfile);
+    fprintf(stderr,"test_data url=%s\n",url);
 #endif
   
     /* Use the open/close mechanism */
