@@ -490,7 +490,6 @@ movetor(NCDAPCOMMON* nccomm,
     CDFnode* xnode = (CDFnode*)nclistget(path,depth);
     OCdatanode reccontent = NULL;
     OCdatanode dimcontent = NULL;
-    OCdatanode fieldcontent = NULL;
     Dapodometer* odom = NULL;
     int hasstringdim = 0;
     DCEsegment* segment;
@@ -506,8 +505,8 @@ movetor(NCDAPCOMMON* nccomm,
     if(ocstat != OC_NOERR) {THROWCHK(ocstat); goto done;}
 
 #ifdef DEBUG2
-fprintf(stderr,"moveto: nctype=%d depth=%d dimindex=%d mode=%s",
-        xnode->nctype, depth,dimindex,oc_data_modestring(mode));
+fprintf(stderr,"moveto: nctype=%d depth=%d dimindex=%d",
+        xnode->nctype, (int)depth, (int)dimindex);
 fprintf(stderr," segment=%s hasstringdim=%d\n",
 		dcetostring((DCEnode*)segment),hasstringdim);
 #endif
@@ -605,7 +604,6 @@ fprintf(stderr," segment=%s hasstringdim=%d\n",
 
 done:
     oc_data_free(conn,dimcontent);
-    oc_data_free(conn,fieldcontent);
     oc_data_free(conn,reccontent);
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
     if(odom) dapodom_free(odom);
@@ -627,8 +625,6 @@ movetofield(NCDAPCOMMON* nccomm,
     size_t fieldindex,gridindex;
     OClink conn = nccomm->oc.conn;
     CDFnode* xnode = (CDFnode*)nclistget(path,depth);
-    OCdatanode reccontent = NULL;
-    OCdatanode dimcontent = NULL;
     OCdatanode fieldcontent = NULL;
     CDFnode* xnext;
     int newdepth;
@@ -675,9 +671,7 @@ movetofield(NCDAPCOMMON* nccomm,
 		     segments);
 
 done:
-    oc_data_free(conn,dimcontent);
     oc_data_free(conn,fieldcontent);
-    oc_data_free(conn,reccontent);
     if(ocstat != OC_NOERR) ncstat = ocerrtoncerr(ocstat);
     return THROW(ncstat);
 }
@@ -903,7 +897,7 @@ slicestring(OClink conn, char* stringmem, DCEslice* slice, struct NCMEMORY* memo
 fprintf(stderr,"moveto: slicestring: string/%lu=%s\n",stringlen,stringmem);
 fprintf(stderr,"slicestring: %lu string=|%s|\n",stringlen,stringmem);
 fprintf(stderr,"slicestring: slice=[%lu:%lu:%lu/%lu]\n",
-slice->first,slice->stride,slice->stop,slice->declsize);
+slice->first,slice->stride,slice->last,slice->declsize);
 #endif
 
     /* Stride across string; if we go past end of string, then pad*/
